#!/bin/bash

AWK="/usr/bin/awk"
BASENAME="/bin/basename"
DATE="/bin/date"
ECHO="/bin/echo"
GREP="/usr/bin/grep"
HOSTNAME="/bin/hostname"
ID="/usr/xpg4/bin/id"
LS="/bin/ls"
SED="/usr/bin/sed"
SORT="/usr/bin/sort"
TOUCH="/bin/touch"
UNAME="/bin/uname"
XARGS="/usr/bin/xargs"

OS="unknown"
OS_VERSION="unknown"

OUTPUT_DIR=""
MODULES_DIR="modules.d"

SUCCESS=0
ERROR=1

function print_error {
   ${ECHO} "Error: $@" >&2
}

function print_warn {
   ${ECHO} "WARNING: $@"
}

function print_info {
   ${ECHO} "INFO: $@"
}

function print_ok {
   ${ECHO} "OK: $@"
}

function print_banner {
   ${ECHO} "[ $@ ]"
}

function print_divider {
   ${ECHO} "==========================================================================================="
}

function print_usage {
  {
     ${ECHO} "Usage: $( ${BASENAME} $0 ) [-h] [-m <modules_dir>] [-o <output_dir>]"
     ${ECHO} "       -m <modules_dir>   Select modules directory"
     ${ECHO} "       -o <output_dir>    Select output directory"
     ${ECHO} "       -h                 Display this usage message"
  } >&2
}

function check_os {
   OPERATING_SYSTEM=$( ${UNAME} -s )   
   OS_REVISION=$( ${UNAME} -r )
   case "${OPERATING_SYSTEM}" in
      "SunOS")	OS="Solaris"
		case "${OS_REVISION}" in
		   "5.10")  OS_VERSION="10" ;;
		   "5.9" )  OS_VERSION="9"  ;;
		   "5.8" )  OS_VERSION="8"  ;;
                   "5.7" )  OS_VERSION="7"  ;;
                   *     )  OS_VERSION="UNSUPPORTED" ;;
                esac ;;
       *     )  OS="UNSUPPORTED" ;;
   esac
   if [ "${OS}" = "UNSUPPORTED" -o "${OS_VERSION}" = "UNSUPPORTED" ]; then
      print_error "Sorry, ${OPERATING_SYSTEM} ${OS_REVISION} is not supported"
      exit ${ERROR}
   elif [ "${OS_VERSION}" -ne "10" ]; then   # temporary....
      print_error "Sorry, only Solaris 10 supported at present"
      exit ${ERROR}
   fi
}

function check_user {
   MY_UID=$( ${ID} -u )
   if [ "${MY_UID}" -ne "0" ]; then
      print_error "This script must be executed as root"
      exit ${ERROR}
   fi
}

function check_output_dir {
   if [ "${OUTPUT_DIR}" != "" ]; then
      if [ ! -e "${OUTPUT_DIR}" ]; then
         print_error "Output directory ${OUTPUT_DIR} does not exist" && exit 1
      elif [ ! -d "${OUTPUT_DIR}" ]; then
         print_error "Output directory ${OUTPUT_DIR} is not a directory" && exit 1
      fi
   fi
}

function check_modules_dir {
   if [ "${MODULES_DIR}" != "" ]; then
      if [ ! -e "${MODULES_DIR}" ]; then
         print_error "Modules directory ${MODULES_DIR} does not exist" && exit 1
      elif [ ! -d "${MODULES_DIR}" ]; then
         print_error "Modules directory ${MODULES_DIR} is not a directory" && exit 1
      fi
   fi
   ${LS} ${MODULES_DIR}/seccheck_[0-9][0-9]_*.sh >/dev/null 2>&1
   if [ "$?" -ne "0" ]; then
      print_error "${MODULES_DIR} contains no seccheck modules" && exit 1
   fi
}

function check_modules {
   print_banner "SECCHECK - Checking all Modules in ${MODULES_DIR}"
   print_divider
   case ${OS_VERSION} in
      "10")
          cd ${MODULES_DIR}
          ${LS} ./seccheck_[0-9][0-9]_*.sh | ${SORT} | ${SORT} -k1.10,1.11n | while read MODULE; do
             ${MODULE}
          done
          ;;
   esac
}

function create_filename {
   OUTPUT_FILENAME="${OUTPUT_DIR}/seccheck-$( ${HOSTNAME} )-$( ${DATE} +"%Y%m%d-%H%M" ).log"
}

function setup_redirection {
   create_filename
   ${TOUCH} ${OUTPUT_FILENAME} >/dev/null 2>&1
   if [ "$?" -ne "0" ]; then
      print_error "Could not create output file" && exit 1
   fi
   exec >${OUTPUT_FILENAME} 2>&1
}

check_os
check_user

while getopts ":hm:o:" OPTION; do
   case ${OPTION} in
      "h") print_usage && exit 0
           ;;
      "m") MODULES_DIR="${OPTARG}"
           ;;
      "o") OUTPUT_DIR="${OPTARG}"
           ;;
      *)   print_usage && exit 1
           ;;
   esac
done

shift $(( ${OPTIND} - 1 ))

if [ "$#" -ne "0" ]; then
   print_usage && exit 1
fi

check_output_dir
check_modules_dir
if [ "${OUTPUT_DIR}" != "" ]; then
   setup_redirection
fi
print_divider
${ECHO} "SCAN STARTED: $( ${DATE} )"
print_divider
check_modules
print_divider
${ECHO} "SCAN FINISHED: $( ${DATE} )"
print_divider

exit 0
